
#include "atomCluster.h"

// Constructor
//
AtomCluster::AtomCluster(int aNum) :
  grid(0), atomNum(aNum), atomCount(0), box(0) {

  atoms = new Atom*[atomNum];

  for (int i=0; i<atomNum; i++) {
    atoms[i] = 0;
  }

  return;
}


// Constructor
//   Allow user to set the search box explicitly
/*
AtomCluster::AtomCluster(int aNum, Box* b) :
  grid(0), atomNum(aNum), atomCount(0), box(0) {

  atoms = new Atom*[atomNum];

  for (int i=0; i<atomNum; i++) {
    atoms[i] = 0;
  }

  Coordinate3D* tempCoord = 0;

  if (box != 0) {
    delete box;
    box = 0;
  }

  box = b->clone();

  return;
}
*/


// Destructor
//
AtomCluster::~AtomCluster() {

  for (int i=0; i<atomCount; i++) {
    if (atoms[i] != 0) {
      delete atoms[i];
    }
  }

  delete atoms;
  
  if (box != 0) {
    delete box;
  }

  return;
}


// getAtomCount
//
int AtomCluster::getAtomCount() {

  return atomCount;
}


// getBox
//   DEBUGGING METHOD
Box* AtomCluster::getBox() {

  return box;
}


// setBox
//  
void AtomCluster::setBox(Box* b) {

  box = b->clone();
  return;
}


// makeGrid
//
AtomGrid* AtomCluster::makeGrid() {

  Coordinate3D* tempCoord1 = box->getOrigin();
  Coordinate3D* tempCoord2 = new Coordinate3D(tempCoord1->getX()-3,
					      tempCoord1->getY()-3,
					      tempCoord1->getZ()-3);
  // XXX - Don't delete tempBox; it is being used within grid
  Box* tempBox = new Box(tempCoord2,
			 box->getWidth()+6,
			 box->getHeight()+6,
			 box->getDepth()+6);
  delete tempCoord2;
  grid = new AtomGrid(tempBox);
  
  for (int i=0; i<atomCount; i++) {
    //printf("  i: %d\n",i);
    grid->addAtom(atoms[i]);
  }

  return grid;
}


// getRandomVolumeCoord3D
//   Returns a coordinate3D that is within the volume
//   defined by the union of atomic Van der Waals radii;
//   takes different amounts of time because initial
//   guesses must be checked against the atom set to
//   guarantee that they lie within the volume
//   MUST DELETE THIS COORDINATE3D AT SOME POINT!!!
Coordinate3D* AtomCluster::getRandomVolumeCoord3D() {

  if (atomCount == 0) {
    return 0;
  }

  if (box == 0) {
    createBox();
  }

  int pointCheck = 0;
  Coordinate3D* tempCoord = 0;

  int i = 0;
  do {
    tempCoord = box->getRandomCoord3D();
    pointCheck = isInVolume(tempCoord);
    i++;
    if (pointCheck == 0) {
      delete tempCoord;
    }
    //printf("  random coordinate: (%f, %f, %f)\n",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
    //printf("  within VDW radius? %d\n",pointCheck);
  } while (pointCheck == 0);

  //printf("  i = %d\n", i);

  return tempCoord;
}


// getRandomNullCoord3D
//   Returns a coordinate3D that is within the volume
//   defined by the union of atomic Van der Waals radii;
//   takes different amounts of time because initial
//   guesses must be checked against the atom set to
//   guarantee that they lie within the volume
//   MUST DELETE THIS COORDINATE3D AT SOME POINT!!!
Coordinate3D* AtomCluster::getRandomNullCoord3D() {

  //printf("=>AtomCluster::getRandomNullCoord3D\n");

  if (atomCount == 0) {
    return 0;
  }

  if (box == 0) {
    createBox();
  }

  int pointCheck = 1;
  Coordinate3D* tempCoord = 0;

  int i = 0;
  do {
    tempCoord = box->getRandomCoord3D();
    pointCheck = isInVolume(tempCoord);
    i++;
    if (pointCheck != 0) {
      delete tempCoord;
    }
    //printf("  random coordinate: (%f, %f, %f)\n",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
    //printf("  within VDW radius? %d\n",pointCheck);
  } while (pointCheck != 0);

  //printf("  i = %d\n", i);

  //printf("<=AtomCluster::getRandomNullCoord3D\n");
  return tempCoord;
}


// getRandomBoxCoord3D
//   Returns a coordinate3D that is within the volume
//   of the box
//   MUST DELETE THIS COORDINATE3D AT SOME POINT!!!
Coordinate3D* AtomCluster::getRandomBoxCoord3D() {

  if (atomCount == 0) {
    return 0;
  }

  if (box == 0) {
    createBox();
  }

  return box->getRandomCoord3D();
}


// addAtom
//
int AtomCluster::addAtom(Atom* atom) {

  //printf("=>addAtom()\n");

  if (atom != 0) {
    atoms[atomCount] = atom;
    atomCount++;
    if (box != 0) {
      delete box;
      box = 0;
    }
    //printf("  atomCount = %d\n",atomCount);
    //printf("<=addAtom(): 1\n");
    return 1;
  }

  //printf("<=addAtom(): 0\n");
  return 0;
}


/*
// getRandomCoord3D
//
Coordinate3D* AtomCluster::getRandomCoord3D() {

return 0;
}
*/


// createBox
//   Generate a box based on the current atomic
//   coordinates.
int AtomCluster::createBox() {

  //printf("=>AtomCluster::createBox\n");

  // box boundaries
  float x1, x2, y1, y2, z1, z2 = 0.0;
  int i = 0;   // counter
  Coordinate3D* tempCoord = 0;

  if (box != 0) {
    delete box;
    box = 0;
  }

  // XXX - CHANGED FOR NULLSPACE TESSELLATION {{{
  //tempCoord = new Coordinate3D(0.0,-15.0,40.0);
  //box = new Box(tempCoord,20.0,35.0,40.0);
  //tempCoord = new Coordinate3D(8.0,-5.0,45.0);
  //box = new Box(tempCoord,7.0,10.0,15.0);
  //tempCoord = new Coordinate3D(-16.0,-15.0,14.0);
  //box = new Box(tempCoord,34.0,30.0,19.0);
  //tempCoord = new Coordinate3D(-16.0,-15.0,19.0);
  //box = new Box(tempCoord,34.0,30.0,12.0);
  //tempCoord = new Coordinate3D(-18.0,-17.0,19.0);
  //box = new Box(tempCoord,38.0,34.0,12.0);
  //delete tempCoord;
  //return 1;
  // }}} XXX - CHANGED FOR NULLSPACE TESSELLATION

  if (atomCount > 0) {
    tempCoord = atoms[i]->getCoordinate3D();
    x1 = x2 = tempCoord->getX();
    y1 = y2 = tempCoord->getY();
    z1 = z2 = tempCoord->getZ();
    i++;
  }
  else {
    //printf("<=AtomCluster::createBox\n");
    return 0;
  }
  
  for (; i<atomCount; i++) {
    tempCoord = atoms[i]->getCoordinate3D();
    if (tempCoord->getX() < x1) {
      x1 = tempCoord->getX();
    }
    if (tempCoord->getX() > x2) {
      x2 = tempCoord->getX();
    }
    if (tempCoord->getY() < y1) {
      y1 = tempCoord->getY();
    }
    if (tempCoord->getY() > y2) {
      y2 = tempCoord->getY();
    }
    if (tempCoord->getZ() < z1) {
      z1 = tempCoord->getZ();
    }
    if (tempCoord->getZ() > z2) {
      z2 = tempCoord->getZ();
    }
  }

  // (+ 4.0) because max VDW radius is 2.0 for both sides
  float width  = x2 - x1 + 4.0;
  float height = y2 - y1 + 4.0;
  float depth  = z2 - z1 + 4.0;
  // Must move origin over by maximum VDW radius, also
  tempCoord = new Coordinate3D(x1-2.0,y1-2.0,z1-2.0);

  box = new Box(tempCoord, width, height, depth);
  delete tempCoord;

  //printf("<=AtomCluster::createBox\n");
  return 1;
}


// isInVolume
//   Takes a 3D point and determines whether it is
//   within the Van der Waals radius of any of the
//   atoms in the cluster
int AtomCluster::isInVolume(Coordinate3D* coord) {

  //printf("=>AtomCluster::isInVolume\n");

  // Retrieve the atoms that are within 2 Angstroms
  //   of the coordinate
  //printf("  atomCount: %d\n",atomCount);
  if (grid == 0) {
    makeGrid();
  }

  Atom* atom = grid->getFirstAtom(coord);
  while (atom != 0) {
    if (atom->isWithinVdwRadius(coord)) {
      //printf("<=AtomCluster::isInVolume\n");
      return 1;
    }
    atom = grid->getNextAtom();
  }

  //printf("<=AtomCluster::isInVolume\n");
  return 0;
}


// isInVolume2
//   Takes a 3D point and determines whether it is
//   within the Van der Waals radius of any of the
//   atoms in the cluster
int AtomCluster::isInVolume2(Coordinate3D* coord) {

  //printf("=>AtomCluster::isInVolume2()\n");

  // Retrieve the atoms that are within 2 Angstroms
  //   of the coordinate
  //printf("  atomCount: %d\n",atomCount);
  for (int i=0; i<atomCount; i++) {
    //printf("  i: %d\n",i);
    // if (within VDW radius) return 1
    if (atoms[i]->isWithinVdwRadius(coord)) {
      //printf("  atom[%d]\n",i);
      //printf("<=AtomCluster::isInVolume(): 1\n");
      return 1;
    }
  }

  //printf("<=AtomCluster::isInVolume2(): 0\n");
  return 0;
}
