
#ifndef ATOMGRID_H
#define ATOMGRID_H

#include "atom.h"
#include "box.h"
#include "coordinate3D.h"

#include <math.h>
#include <stdio.h>


class AtomNode {

 public:
  AtomNode(Atom* a);
  ~AtomNode();

  AtomNode* getNext();
  Atom* getAtom();
  int setNext(AtomNode* node);
  int printAtomNode(FILE* outfile);

 private:
  Atom* atom;
  AtomNode* next;

};



class AtomList {

 public:
  AtomList();
  ~AtomList();

  Atom* getFirstAtom();
  Atom* getNextAtom();
  int addAtom(Atom* atom);
  int printAtomList(FILE* outfile);

 private:
  AtomNode* head;
  AtomNode* current;

};



class AtomGridCell {

 public:
  AtomGridCell(int a, int b, int c);
  ~AtomGridCell();

  int getX();
  int getY();
  int getZ();

 private:
  int x;
  int y;
  int z;

};



class AtomGrid {

 public:
  AtomGrid(Box* box);
  ~AtomGrid();

  Atom* getFirstAtom(Coordinate3D* c);
  Atom* getNextAtom();
  int addAtom(Atom* atom);
  int getFocusIndex();
  int printAtomGrid(FILE* outfile);

 private:
  AtomList* getAtomList(AtomGridCell* cell);
  int setAtomList(AtomGridCell* cell, AtomList* list);
  int divide();
  AtomGridCell* coordinate3DToCell(Coordinate3D* coord);
  int overlapsSide(Coordinate3D* coord, int side);
  int setFocusCells(int numSides, int x, int y, int z);

 private:
  AtomList**** grid;            // 3D array of atom bins
  AtomGridCell* focusCells[8];  // Cells to search through
  Coordinate3D* origin;         // Origin of box
  int focusIndex;       // Which cell in focusCells is being searched
  int width;            // Width of box
  int height;           // Height of box
  int depth;            // Depth of box
  float distanceUnit;   // Cell side length (MUST BE > 4)

  enum {XP, XN, YP, YN, ZP, ZN};   // Positive or negative directions along axes

};

#endif
