
#ifndef GENERICLIST_H
#define GENERICLIST_H

//#include "genericList.c"

#include <stdio.h>


/**
 * GenericListNode.  Node for a GenericList.
 */

template<class T> class GenericListNode {

 public:
  /** Constructor. */
  GenericListNode(T p);

  /** Destructor. */
  ~GenericListNode();

  /**
   * Returns the object held by this node.
   * @return class T: object held by this node.
   */
  T getPackage();

  /**
   * Returns the previous node in the list.
   * @return GenericListNode*: previous node in the list.
   */
  GenericListNode* getPrevious();

  /**
   * Returns the next node in the list.
   * @return GenericListNode*: next node in the list.
   */
  GenericListNode* getNext();

  /**
   * Sets the previous node in the list.
   * @param p GenericListNode*: previous node in the list.
   * @return int: 0 on fail, 1 on success.
   */
  int setPrevious(GenericListNode* p);

  /**
   * Sets the next node in the list.
   * @param n GenericListNode*: next node in the list.
   * @return int: 0 on fail, 1 on success.
   */
  int setNext(GenericListNode* n);

 private:
  T package;
  GenericListNode* previous;
  GenericListNode* next;

};


/**
 * GenericList. List container that has queue and stack functionality.
 */

template<class T> class GenericList {

 public:

  /** Constructor. */
  GenericList();

  /** Destructor. Does not delete objects stored in the list. */
  ~GenericList();


  /**
   * Returns the first element of the list.
   * @return class T: first element of the list.
   */
  T getFirst();

  /**
   * Returns the next element of the list.
   * @return class T: next element of the list.
   */
  T getNext();

  /**
   * Returns the length of the list.
   * @return int: length of the list.
   */
  int getLength();

  /**
   * Removes and returns the first element of the list.  Queue functionality.
   * @return class T: first element of the list.
   * @see push(T t)
   */
  T front();

  /**
   * Removes and returns the last element of the list.  Stack functionality.
   * @return class T: last element of the list.
   * @see push(T t)
   */
  T pop();

  /**
   * Places a new element at the end of the list.
   * @param t class T: element to add to the list.
   * @return int: 0 on fail, 1 on success.
   * @see front()
   * @see pop()
   */
  int push(T t);

  /**
   * Return the element at the given index in the list.  Sets current node to this node.
   * @param index int: index into the list.
   * @return class T: element at this index.
   */
  T returnElement(int index);

  /**
   * Remove the element at the given index in the list.
   * @param index int: index into the list.
   * @return class T: element at this index.
   */
  T removeElement(int index);

  /**
   * Clone the list.
   * @return GenericList<T>*: Clone of list.
   */
  GenericList<T>* clone();

  /**
   * Debugging method. Prints the state of the list.
   * @return int: 0 on fail, 1 on success.
   */
  int printList();

 private:
  GenericListNode<T>* head;
  GenericListNode<T>* tail;
  GenericListNode<T>* current;
  int length;

};

#endif
