
#include "atom.h"
#include "atomCluster.h"
#include "baseGraph.h"
#include "box.h"
#include "connectionMatrix.h"
#include "coordinate3D.h"
#include "element.h"
#include "elementBuilder.h"
#include "neuronList.h"
#include "pdbReader.h"
#include "tessellationScriptReader.h"
#include "tessellator.h"
#include "tetrahedronGraph.h"
#include "triangleGraph.h"

#include <stdio.h>
#include <stdlib.h>

struct Params {

  // Commands.
  int   readScript;
  int   tessellate;
  int   prune;
  int   surface;
  int   movePrune;
  int   moveSurface;

  // Parameters.
  char pdbFileName[1024];
  char psfFileName[1024];
  char tessellationFileName[1024];
  int   hebbian;
  int   startStep;   // Indexed at 1, not 0.
  int   numSteps;
  int   printStep;
  int   numVertices;
  float vertexRadius;
  float boxOriginX;
  float boxOriginY;
  float boxOriginZ;
  float boxWidth;
  float boxHeight;
  float boxDepth;

  void init() {
    readScript = 0;
    tessellate = 0;
    prune = 0;
    surface = 0;
    movePrune = 0;
    moveSurface = 0;
    pdbFileName[0] = '\0';
    psfFileName[0] = '\0';
    tessellationFileName[0] = '\0';
    hebbian   = 1;
    startStep = 1;
    numSteps  = 0;
    printStep = 0;
    numVertices = 0;
    vertexRadius = 1.4;  // Default for H2O
    boxOriginX  = 0.0;
    boxOriginY  = 0.0;
    boxOriginZ  = 0.0;
    boxWidth    = 0.0;
    boxHeight   = 0.0;
    boxDepth    = 0.0;
    return;
  }

};


int readParamInt(char* line, char* token, int* num) {

  char *substring, read_format[100];

  if( (substring=strstr(line,token)) == NULL )  {
    return 0;
  }
  
  sprintf(read_format,"%s %%d",token);
  printf("%s\n",token);

  if( sscanf(substring,read_format,num) != 1 ) {
    printf("Error: wrong configuration line - %s\n",line);
    return 0;
  }

  printf("  %d\n",*num);

  return 1;
}


int readParamFloat(char* line, char* token, float* num) {

  char *substring, read_format[100];

  if( (substring=strstr(line,token)) == NULL )  {
    return 0;
  }
  
  sprintf(read_format,"%s %%f",token);
  printf("%s\n",token);

  if( sscanf(substring,read_format,num) != 1 ) {
    printf("Error: wrong configuration line - %s\n",line);
    return 0;
  }

  printf("  %f\n",*num);

  return 1;
}


int readParamString(char* line, char* token, char* word) {

  char *substring, read_format[100];

  if( (substring=strstr(line,token)) == NULL )  {
    return 0;
  }
  
  sprintf(read_format,"%s %%s",token);
  printf("%s\n",token);

  if( sscanf(substring,read_format,word) != 1 ) {
    printf("Error: wrong configuration line - %s\n",line);
    return 0;
  }

  printf("  %s\n",word);

  return 1;
}


int readParameterFile(FILE* paramFile, Params* params) {

  char s[100];
  int i_dum;

  fgets(s, 90, paramFile);
  do {
    //fgets(s, 90, paramFile);
    //printf("Yo\n");
    i_dum = 0;
    i_dum += readParamInt(s,"READ_SCRIPT",&params->readScript);
    i_dum += readParamInt(s,"TESSELLATE",&params->tessellate);
    i_dum += readParamInt(s,"PRUNE",&params->prune);
    i_dum += readParamInt(s,"SURFACE",&params->surface);
    i_dum += readParamInt(s,"MOVE_PRUNE",&params->movePrune);
    i_dum += readParamInt(s,"MOVE_SURFACE",&params->moveSurface);
    i_dum += readParamString(s,"PDB_FILE_NAME",params->pdbFileName);
    i_dum += readParamString(s,"PSF_FILE__NAME",params->psfFileName);
    i_dum += readParamString(s,"TESSELLATION_FILE_NAME",params->tessellationFileName);
    i_dum += readParamInt(s,"HEBBIAN",&params->hebbian);
    i_dum += readParamInt(s,"START_STEP",&params->startStep);
    i_dum += readParamInt(s,"NUM_STEPS", &params->numSteps);
    i_dum += readParamInt(s,"PRINT_STEP",&params->printStep);
    i_dum += readParamInt(s,"NUM_VERTICES",&params->numVertices);
    i_dum += readParamFloat(s,"VERTEX_RADIUS",&params->vertexRadius);
    i_dum += readParamFloat(s,"BOX_ORIGIN_X",&params->boxOriginX);
    i_dum += readParamFloat(s,"BOX_ORIGIN_Y",&params->boxOriginY);
    i_dum += readParamFloat(s,"BOX_ORIGIN_Z",&params->boxOriginZ);
    i_dum += readParamFloat(s,"BOX_WIDTH", &params->boxWidth);
    i_dum += readParamFloat(s,"BOX_HEIGHT",&params->boxHeight);
    i_dum += readParamFloat(s,"BOX_DEPTH", &params->boxDepth);
    fgets(s, 90, paramFile);
  } while( !feof(paramFile) );

  return 1;
}


/**
 *
 * @return int: 0 on fail, 1 on success.
 */
int validateParams(Params* params) {


  return 1;
}


/**
 *
 * @return int: 0 on fail, 1 on success.
 */
int postProcessing(Params* params) {



  return 1;
}


/**
 *
 * @return int: 0 on fail, 1 on success.
 */
int colorProcessing(Params* params) {


  return 1;
}


/**
 * Main
 * @return int: 0 on fail, 1 on success.
 */
int main (int argc, char** argv) {
  
  printf("\n");
  printf("=========================================================================\n");
  printf(" TESSELLATE: A program for tessellating space around biological          \n");
  printf("   macromolecules.                                                       \n");
  printf(" VERSION: 1.0                                                            \n");
  printf(" AUTHOR:  John Eargle                                                    \n");
  printf(" (C) Copyright 2005 Luthey-Schulten Group and The Board of Trustees of   \n");
  printf("       the University of Illinois.                                       \n");
  printf("=========================================================================\n");
  printf("\n");

  // Check command line arguments.
  if (argc != 2) {
    printf("Error: Wrong number of arguments\n");
    exit(1);
  }

  // Read in parameter file.
  Params params;  // = new Params();
  params.init();
  char* paramFileName = argv[1];
  FILE* paramFile = fopen(paramFileName, "r");
  if (paramFile == NULL) {
    printf("Error: Cannot open %s\n", paramFileName);
    exit(1);
  }
  readParameterFile(paramFile, &params);
  fclose(paramFile);

  // Validate params.
  if ( validateParams(&params) == 0 ) {
    printf("Error: Invalid parameters\n");
  }
  
  ElementBuilder* elements = new ElementBuilder();

  // Read pdb file.
  PdbReader* pdbReader = new PdbReader();
  pdbReader->setFilename(params.pdbFileName);
  AtomCluster* atoms = pdbReader->getAtomCluster(0,elements);
  Coordinate3D* boxOrigin = new Coordinate3D(params.boxOriginX,
					     params.boxOriginY,
					     params.boxOriginZ);
  Box* tessBox = new Box(boxOrigin, params.boxWidth, params.boxHeight, params.boxDepth);
  atoms->setBox(tessBox);

  // Setup tessellator.
  Tessellator* tessellator = 0;
  BaseGraph* graph = 0;
  if (params.readScript == 1) {
    TessellationScriptReader* tsr = new TessellationScriptReader();
    tsr->setFilename(params.tessellationFileName);
    graph = tsr->getBaseGraph();
    tessellator = new Tessellator(atoms, graph->getVertexCount(), params.vertexRadius, graph->getCoords(), graph->getConnectionMatrix());
    if (graph->getVertexCount() != params.numVertices) {
      printf("Error: different number of vertices in script file and config file\n");
    }
  }
  else {
    tessellator = new Tessellator(atoms, params.numVertices, params.vertexRadius);
  }

  if (params.startStep > 1) {
    tessellator->setStepNum(params.startStep);
  }
  
  char outFileName[1024] = "tess";
  FILE* outFile;
  //int tempLen = strLen(params.pdbFileName);
  //strncpy(outFileName,params.pdbFileName,tempLen);
  //outFileName[tempLen] = '\0';
  //char* outFileNameSuffix;
  int i;
  int j=1;
  for (i=params.startStep; i<=params.numSteps; i++) {
    tessellator->stepTessellation();
    if (tessellator->areVerticesStable() == 1) {
      printf("Vertices are stable: step %d\n",i);
      break;
    }
    if (i%1000 == 0) {
      printf("  step: %d, adds: %d, stability: %f\n",i,tessellator->getAddTracker(), tessellator->getVertexStability());
      //tessellator->getVertexStability();
      tessellator->resetAddTracker();
    }
    if (i%params.printStep == 0) {
      //itoa(i,outFileNameSuffix,10);
      if (j > 9) {
	const char* outFileNameSuffix = intToString(j,2);
	outFileName[4] = '\0';
	strncat(outFileName,outFileNameSuffix,2);
	outFileName[6] = '\0';
	strncat(outFileName,".tcl",4);
	outFileName[10] = '\0';
	delete outFileNameSuffix;
      }
      else {
	const char* outFileNameSuffix = intToString(j,1);
	outFileName[4] = '\0';
	strncat(outFileName,"0",1);
	outFileName[5] = '\0';
	strncat(outFileName,outFileNameSuffix,1);
	outFileName[6] = '\0';
	strncat(outFileName,".tcl",4);
	outFileName[10] = '\0';
	delete outFileNameSuffix;
      }
      outFile = fopen(outFileName,"w");
      tessellator->printTessellationScript(outFile);
      fclose(outFile);
      printf("Printing file: %s\n",outFileName);
      j++;
    }
  }

  //printf("Starting edgeFinalization test\n");
  //tessellator->finalizeConnections();
  //printf("Ending edgeFinalization test\n");

  // Final printout.
  outFileName[0] = '\0';
  strncat(outFileName,"tessFinal.tcl",13);
  outFileName[13] = '\0';
  outFile = fopen(outFileName,"w");
  tessellator->printTessellationScript(outFile);
  fclose(outFile);
  printf("Printing file: %s\n",outFileName);

  
  // Tessellation post-processing.
  if (params.prune + params.surface > 0) {
    TessellationScriptReader* tessReader = new TessellationScriptReader();
    tessReader->setFilename(outFileName);
    BaseGraph* tempGraph = tessReader->getBaseGraph();
    //tsr->setPath("./");
    // Remove long edges.
    if (params.prune != 0) {
      tempGraph->deactivateLongEdges();
      outFile = fopen("prune.tcl","w");
      tempGraph->printTessellationScript(outFile);
      fclose(outFile);
    }
    
    // Surface
    if (params.surface != 0) {
      TriangleGraph* triangleGraph = tempGraph->getTriangleGraph();
      //tGraph2->print();
      outFile = fopen("surface.tcl","w");
      triangleGraph->printTessellationScript(outFile);
      fclose(outFile);

      printf("Starting TriangleEdgeTree test\n");
      triangleGraph->addEdges();
      printf("Ending TriangleEdgeTree test\n");

      printf("Starting TriangleGraph print2 test\n");
      triangleGraph->print2();
      printf("Ending TriangleGraph print2 test\n");

      printf("Starting TetrahedronGraph creation test\n");
      TetrahedronGraph* tetrahedronGraph = triangleGraph->getTetrahedronGraph();
      printf("Ending TetrahedronGraph creation test\n");

      printf("Starting TetrahedronGraph volume test\n");
      float v = tetrahedronGraph->getVolume();
      printf("  tetrahedronGraph volume = %3.3f\n",v);
      printf("Ending TriangleGraph volume test\n");

      printf("Starting TetrahedronEdgeTree test\n");
      tetrahedronGraph->addEdges();
      printf("Ending TetrahedronEdgeTree test\n");

      printf("Starting TetrahedronGraph triangle marking test\n");
      tetrahedronGraph->setTriangleTypes();
      printf("Ending TetrahedronGraph triangle marking test\n");

      printf("Starting TriangleGraph internal deactivation test\n");
      triangleGraph->deactivateInternal();
      outFile = fopen("surfaceArea.tcl","w");
      triangleGraph->printTessellationScript(outFile);
      fclose(outFile);
      printf("Ending TriangleGraph internal deactivation test\n");

      printf("Starting TriangleGraph surface area test\n");
      //float sa = triangleGraph->getSurfaceArea();
      //printf("  triangleGraph surface area = %3.3f\n",sa);
      printf("Ending TriangleGraph surface area test\n");
    }
  }


  // Tessellation move color processing.
  if (params.readScript != 0) {
    if (params.movePrune + params.moveSurface > 0) {
      TessellationScriptReader* tessReader = new TessellationScriptReader();
      tessReader->setFilename(outFileName);
      BaseGraph* tempGraph = tessReader->getBaseGraph();
      tempGraph->compareVertexPositions(graph);
      // MovePrune
      if (params.movePrune != 0) {
	tempGraph->deactivateLongEdges();
	outFile = fopen("colorPrune.tcl","w");
	tempGraph->printTessellationScript(outFile,1);
	fclose(outFile);
      }
      
      // MoveSurface
      if (params.surface != 0) {
	TriangleGraph* triangleGraph = tempGraph->getTriangleGraph();
	outFile = fopen("colorSurface.tcl","w");
	triangleGraph->printTessellationScript(outFile,1);
	fclose(outFile);
      }
    }
  }

  return 0;
}
