
#ifndef TESSELLATIONSCRIPTREADER_H
#define TESSELLATIONSCRIPTREADER_H

#include "baseGraph.h"
#include "coordinate3D.h"
#include "genericList.h"
#include "typeConvert.h"

#include <stdio.h>
#include <string.h>

class TessellationScriptReader {

  /**
   * Edge coordinates.
   */
  struct EdgeCoords {
    Coordinate3D* coord1;
    Coordinate3D* coord2;
    
    EdgeCoords(Coordinate3D* c1, Coordinate3D* c2) :
      coord1(c1), coord2(c2) {}
  };

 public:
  /**
   * Constructor.
   */
  TessellationScriptReader();

  /**
   * Destructor.
   */
  ~TessellationScriptReader();

  /**
   */
  int setFilename(char* fn);

  /**
   */
  int setPath(char* p);

  /**
   */
  BaseGraph* getBaseGraph();


 private:
  /**
   */
  int getVertices(BaseGraph* graph, int vertexCount);

  /**
   */
  int getEdges(BaseGraph* graph);

  /**
   */
  int getVertexCount();
  
  /**
   */
  EdgeCoords* getEdgeCoordsFromString(char* edgeString);

  /**
   */
  Coordinate3D* getCoord3DFromString(char* coordString);

  /**
   */
  int checkFullName();

  /**
   */
  int checkTessellationFile(char* file);

  /**
   */
  int setNameFromFilename();
  
 private:
  char* filename;
  char* path;      // Path to the file
  char* fullName;

};

#endif
