
#include "atom.h"
#include "atomCluster.h"
#include "box.h"
#include "connectionMatrix.h"
#include "coordinate3D.h"
#include "element.h"
#include "elementBuilder.h"
#include "neuronList.h"
#include "pdbReader.h"
#include "tessellator.h"

#include <stdio.h>
#include <stdlib.h>


int main (int argc, char** argv) {
  
  Coordinate3D* tempCoord;


  // Atom
  printf("*** Atom Test ***\n");

  ElementBuilder* eb = new ElementBuilder();

  Coordinate3D* coord1 = new Coordinate3D(0.0,0.0,0.0);
  Coordinate3D* coord2 = new Coordinate3D(0.0,0.0,1.71);
  Coordinate3D* coord3 = new Coordinate3D(0.0,0.0,1.70);
  Atom* atom1 = new Atom(eb, "C", coord1);
  
  printf("within VDW radius? %d\n",atom1->isWithinVdwRadius(coord1));
  printf("within VDW radius? %d\n",atom1->isWithinVdwRadius(coord2));
  printf("within VDW radius? %d\n",atom1->isWithinVdwRadius(coord3));

  printf("\n\n");


  // Box
  printf("*** Box Test ***\n");
  Box* box = new Box(coord1, 2, 2, 2);
  printf("volume: %3.2f\n",box->getVolume());
  tempCoord = box->getRandomCoord3D();
  printf("random coordinate: (%f, %f, %f)\n",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
  printf("within VDW radius? %d\n",atom1->isWithinVdwRadius(tempCoord));
  delete tempCoord;
  tempCoord = box->getRandomCoord3D();
  printf("random coordinate: (%f, %f, %f)\n",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
  printf("within VDW radius? %d\n",atom1->isWithinVdwRadius(tempCoord));
  delete tempCoord;
  tempCoord = box->getRandomCoord3D();
  printf("random coordinate: (%f, %f, %f)\n",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
  printf("within VDW radius? %d\n",atom1->isWithinVdwRadius(tempCoord));

  printf("\n\n");

  
  // AtomCluster
  printf("*** AtomCluster Test ***\n");
  AtomCluster* ac = new AtomCluster(4);
  Coordinate3D* coord4 = new Coordinate3D(3.0,0.0,0.0);
  Coordinate3D* coord5 = new Coordinate3D(0.0,3.0,0.0);
  Coordinate3D* coord6 = new Coordinate3D(3.0,3.0,0.0);
  Atom* atom2 = new Atom(eb, "C", coord4);
  Atom* atom3 = new Atom(eb, "C", coord5);
  Atom* atom4 = new Atom(eb, "C", coord6);
  
  /*

  ac->addAtom(atom1);
  ac->addAtom(atom2);
  ac->addAtom(atom3);
  ac->addAtom(atom4);
  
  tempCoord = ac->getRandomVolumeCoord3D();
  printf("good coordinate: (%f, %f, %f)\n",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
  tempCoord = ac->getRandomVolumeCoord3D();
  printf("good coordinate: (%f, %f, %f)\n",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
  tempCoord = ac->getRandomVolumeCoord3D();
  printf("good coordinate: (%f, %f, %f)\n",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
  tempCoord = ac->getRandomVolumeCoord3D();
  printf("good coordinate: (%f, %f, %f)\n",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
  tempCoord = ac->getRandomVolumeCoord3D();
  printf("good coordinate: (%f, %f, %f)\n",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());

  printf("\n\n");
  */


  
  // PdbReader
  printf("*** PdbReader Test ***\n");
  PdbReader* pdbReader = new PdbReader();
  pdbReader->setPath("./Test/AspAARS/");
  //pdbReader->setFilename("hisHhisF.pdb");
  //pdbReader->setFilename("side_hole_align.pdb");
  //pdbReader->setFilename("RCcytOnly_ox_eq-heme.pdb");
  //pdbReader->setFilename("RCcytOnly_ox_eq_elev_3A-heme.pdb");
  //pdbReader->setFilename("RCcytOnly_ox_eq_elev_10A-heme.pdb");
  //pdbReader->setFilename("RCcytOnly_ox_eq_pull_24A-heme.pdb");
  pdbReader->setFilename("1asy.pdb");
  //pdbReader->setFilename("1a9x-AB.pdb");
  //pdbReader->setFilename("1c0a-prot.pdb");
  AtomCluster* ac2 = pdbReader->getAtomCluster(0, eb);
  //tempCoord = new Coordinate3D(0.0,-15.0,40.0);
  //box = new Box(tempCoord,20.0,35.0,40.0);
  Coordinate3D* tempCoord5 = new Coordinate3D(44.0,32.0,-27.0);
  Box* box5 = new Box(tempCoord5,15.0,16.0,21.0);
  ac2->setBox(box5);
  delete tempCoord5;
  delete box5;
  
  for (int i=0; i<100; i++) {
    tempCoord = ac2->getRandomVolumeCoord3D();
    //printf("good coordinate: (%f, %f, %f)\n",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
    delete tempCoord;
  }
  
  printf("\n\n");
  

  
  // ConnectionMatrix
  printf("*** ConnectionMatrix Test ***\n");
  ConnectionMatrix* cm = new ConnectionMatrix(5);
  for (int i=0; i<5; i++) {
    for (int j=0; j<5; j++) {
      printf("%3d ",cm->getValue(i,j));
    }
    printf("\n");
  }
  printf("\n");

  cm->setValue(0,1,3);
  cm->setValue(1,2,2);
  cm->setValue(2,4,1);
  cm->setValue(4,3,0);
  cm->setValue(0,3,3);
  cm->setValue(1,3,2);
  //cm->setValue(3,4,1);
  for (int i=0; i<5; i++) {
    for (int j=0; j<5; j++) {
      printf("%3d ",cm->getValue(i,j));
    }
    printf("\n");
  }
  printf("\n");

  cm->incrementValues();
  for (int i=0; i<5; i++) {
    for (int j=0; j<5; j++) {
      printf("%3d ",cm->getValue(i,j));
    }
    printf("\n");
  }
  printf("\n");

  cm->incrementValues();
  for (int i=0; i<5; i++) {
    for (int j=0; j<5; j++) {
      printf("%3d ",cm->getValue(i,j));
    }
    printf("\n");
  }
  printf("\n");

  cm->incrementValues();
  for (int i=0; i<5; i++) {
    for (int j=0; j<5; j++) {
      printf("%3d ",cm->getValue(i,j));
    }
    printf("\n");
  }
  printf("\n");

  cm->incrementValues();
  for (int i=0; i<5; i++) {
    for (int j=0; j<5; j++) {
      printf("%3d ",cm->getValue(i,j));
    }
    printf("\n");
  }
  printf("\n");

  cm->incrementValues();
  for (int i=0; i<5; i++) {
    for (int j=0; j<5; j++) {
      printf("%3d ",cm->getValue(i,j));
    }
    printf("\n");
  }
  printf("\n");

  cm->removeIfGreaterThan(6.3);
  printf("removeIfGreaterThan(6.3)\n");
  for (int i=0; i<5; i++) {
    for (int j=0; j<5; j++) {
      printf("%3d ",cm->getValue(i,j));
    }
    printf("\n");
  }

  printf("\n\n");


  // NeuronList
  printf("*** NeuronList Test ***\n");
  Coordinate3D** coordArray = new Coordinate3D*[3];
  coordArray[0] = coord6;
  coordArray[1] = coord5;
  coordArray[2] = coord4;
  Coordinate3D* focus = new Coordinate3D(6.0, 6.0, 6.0);
  NeuronList* neuronList = new NeuronList(coordArray, 3, focus);
  int index1 = neuronList->getFirstIndex();
  int index2 = neuronList->getSecondIndex();
  printf("  index1 = %d\n",index1);
  printf("  index2 = %d\n",index2);
  for (int i=0; i<3; i++) {
    printf("  coordArray[%d]: (%3.2f, %3.2f, %3.2f)\n",i,coordArray[i]->getX(),coordArray[i]->getY(),coordArray[i]->getZ());
  }
  printf("  focus: (%3.2f, %3.2f, %3.2f)\n",focus->getX(),focus->getY(),focus->getZ());
  neuronList->contract(0.3, 0.6);
  printf("\ncontract\n");
  for (int i=0; i<3; i++) {
    printf("  coordArray[%d]: (%3.2f, %3.2f, %3.2f)\n",i,coordArray[i]->getX(),coordArray[i]->getY(),coordArray[i]->getZ());
  }
  delete neuronList;

  printf("\n\n");
  

  /*
  // AtomGrid
  printf("*** AtomGrid Test ***\n");
  AtomGrid* grid = new AtomGrid(ac->getBox());
  printf("   Hey1\n");
  grid->addAtom(atom1);
  printf("   Hey2\n");
  grid->addAtom(atom2);
  printf("   Hey3\n");
  grid->addAtom(atom3);
  printf("   Hey4\n");
  grid->addAtom(atom4);
  printf("   Hey5\n");
  Atom* atom5 = grid->getFirstAtom(new Coordinate3D(0.1,3.1,0.1));
  Coordinate3D* foo1 = atom5->getCoordinate3D();
  printf("   x: %3.2f, y: %3.2f, z: %3.2f\n",foo1->getX(),foo1->getY(),foo1->getZ());
  Atom* atom6 = grid->getNextAtom();
  if (atom6 == 0) {
    printf("  No next atom\n");
  }
  else {
    Coordinate3D* foo2 = atom6->getCoordinate3D();
    printf("  Next atom - \n");
    printf("     x: %3.2f, y: %3.2f, z: %3.2f\n",foo2->getX(),foo2->getY(),foo2->getZ());
  }

  printf("\n\n");
  AtomGrid* grid2 = ac2->makeGrid();

  printf("\n");
  
  Coordinate3D* bar = 0;
  Atom* baz = 0;
  Coordinate3D* bap = 0;
  for (int i=0; i<10; i++) {
    printf("i = %d\n",i);
    bar = ac2->getRandomBoxCoord3D();
    printf("  bar = (%3.3f,%3.3f,%3.3f)\n",bar->getX(),bar->getY(),bar->getZ());
    //printf("bl33p\n");
    baz = grid2->getFirstAtom(bar);
    //printf("blarg\n");
    while (baz != 0) {
      //printf("blah\n");
      //printf("  focusIndex = %d\n",grid2->getFocusIndex());
      bap = baz->getCoordinate3D();
      printf("***   bap = (%3.3f,%3.3f,%3.3f)\n",bap->getX(),bap->getY(),bap->getZ());
      baz = grid2->getNextAtom();
    }
    //printf("blah2\n");
    printf("\n");
  }
  printf("\n");
  grid2->printAtomGrid(stdout);

  printf("\n\n");
  */


  
  // Tessellator
  printf("*** Tessellator Test ***\n");
  Tessellator* tessellator = new Tessellator(ac2, 2000);
  //Tessellator* tessellator = new Tessellator(ac2, 50);
  FILE* out = fopen("tess1.tcl","w");
  tessellator->printTessellationScript(out);
  fclose(out);
  for (int i=1; i<=800000; i++) {
    tessellator->stepTessellation();
    if (i == 10) {
      out = fopen("tess2.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    if (i == 100) {
      out = fopen("tess3.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 1000) {
      out = fopen("tess4.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 10000) {
      out = fopen("tess5.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 25000) {
      out = fopen("tess6.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 50000) {
      out = fopen("tess7.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 75000) {
      out = fopen("tess8.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 100000) {
      out = fopen("tess9.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 125000) {
      out = fopen("tess10.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 150000) {
      out = fopen("tess11.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 175000) {
      out = fopen("tess12.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 200000) {
      out = fopen("tess13.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 225000) {
      out = fopen("tess14.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 250000) {
      out = fopen("tess15.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 275000) {
      out = fopen("tess16.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 300000) {
      out = fopen("tess17.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 325000) {
      out = fopen("tess18.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 350000) {
      out = fopen("tess19.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 375000) {
      out = fopen("tess20.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 400000) {
      out = fopen("tess21.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 450000) {
      out = fopen("tess22.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 500000) {
      out = fopen("tess23.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 650000) {
      out = fopen("tess24.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 600000) {
      out = fopen("tess25.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 750000) {
      out = fopen("tess26.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    else if (i == 800000) {
      out = fopen("tess27.tcl","w");
      tessellator->printTessellationScript(out);
      fclose(out);
    }
    if (i%1000 == 0) {
      printf(" -- %d, adds: %d\n", i, tessellator->getAddTracker());
      tessellator->resetAddTracker();
      if (tessellator->areVerticesStable() == 1) {
	out = fopen("tess11.tcl","w");
	tessellator->printTessellationScript(out);
	fclose(out);
	printf(" -- %d, Simulated Annealing Completed\n", i);
	break;
      }
    }
  }

  printf("\n\n");
  


  return 0;
}
